// Copyright (C) 2011 INRIA Sophia Antipolis Méditerranée
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef __ABSTRACTROUTINGSTRATEGY_H_
#define __ABSTRACTROUTINGSTRATEGY_H_
#include <OverlayKey.h>
#include <NodeHandle.h>
#include "Synapse.h"
#include "SynapseMessages_m.h"

/**
 * @file AbstractRoutingStrategy.h
 * @author Vincenzo Ciancaglini
 * 
 * Base class for the routing strategy
 */
namespace oversim 
{

class AbstractRoutingStrategy {

friend class SynapseController;

public:
	virtual void AbstractRoutingStrategy(){};
	virtual void ~AbstractRoutingStrategy(){};

protected:
	virtual std::vector<NetworkIdRef> getConnectedOverlay()=0;
	virtual std::vector<NetworkIdRef> getDirectOvelay()=0;
	virtual OverlayKey initNewRequest(OverlayKey key) { return OverlayKey::random(); };
	virtual bool isOpportunistic() { return false; };
	
	virtual bool routingFinished(OverlayKey key, OverlayKey requestID) {return true;};
	virtual RoutingStrategyInformation getRotuingState(OverlayKey key, OverlayKey requestID, RoutingStrategyInformation previousState) = 0;
	virtual SynapseRequestCall* createSynapseRequest(OverlayKey key, OverlayKey requestID) = 0;
};
} //namespace
	
#endif