// Copyright (C) 2011 INRIA Sophia Antipolis Méditerranée
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef __ISYNAPSEBASEOVERLAY_H_
#define __ISYNAPSEBASEOVERLAY_H_
#include <OverlayKey.h>
#include <TransportAddress.h>
/**
 * @file ISynapseBaseOverlay.h
 * @author Vincenzo Ciancaglini
 * 
 * Adds some methods to the BaseOverlay class.
 * They are here so that we don't have to dig into BaseOverlay code and keep things clean
 */

class ISynapseBaseOverlay {
public:
	virtual ~ISynapseBaseOverlay(){};
	// Getter and setter for a node's network ID
	virtual void setNetworkID(const OverlayKey &netID){};
	virtual const OverlayKey & getNetworkID(){ return OverlayKey::UNSPECIFIED_KEY; };
	virtual void addSecondaryNetworkID(const OverlayKey& netID){};
	virtual void removeSecondaryNetworkID(const OverlayKey& netID){};
//	virtual std::vector<std::string> getSecondaryNetIDs(){ return std::vector<std::string>(); };
	
	// To be specialized for each overlay (in case).
	virtual void postInitializeOverlay(){};
	// This has to be modified in each overlay instead of bootstrapList->getBootstrapNode()
	virtual TransportAddress getBootstrapNode(){ return TransportAddress::UNSPECIFIED_NODE; };
};
#endif