// Copyright (C) 2011 INRIA Sophia Antipolis Méditerranée
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//

#ifndef __REVERSENETWORKTABLE_H_
#define __REVERSENETWORKTABLE_H_
#include <OverlayKey.h>
#include <NodeHandle.h>
#include "Synapse.h"
#include <omnetpp.h>
#include <csimplemodule.h>
#include <InitStages.h>

/**
 * @file ReverseNetworkTable.h
 * @author Vincenzo Ciancaglini
 * 
 * Handles the synapse routing tables to foreign overlays
 */
namespace oversim 
{

class ReverseNetworkTable : public cSimpleModule {
	friend class SynapseController;
public:
	ReverseNetworkTable(){};
	
	virtual ~ReverseNetworkTable();
	
	virtual int numInitStages() const
	{
        return MAX_STAGE_OVERLAY + 1;
	}
	virtual void initialize(int stage);
	virtual void handleMessage(cMessage* msg);
	
protected:
	//Extracts the networkID information from a nodehandle and adds it in the correspondent sets
	void addNode(NodeHandle& node);
	//Remove a node entry form the sets where it is present
	void removeNode(NodeHandle& node);
	//Get a random node from a given network sets
	const TransportAddress& getRandomNodeForNetwork(oversim::NetworkIdRef netID);
	//Move the node up in the ordered vector.
	void refreshNode(NodeHandle& node);
private:
	int synapsePort;
	// Node sets stored per networkID
	SynapseNodeMap nodeMap;
	
	// Pointers to element in the nodeMap, sorted by latest ping time.
	NodeIteratorMap sortedNodes;
	
	SynapseNodeSet *getNodeSetForNetID(NetworkIdRef netID, bool createIfNonExistent);
	
	NodeIteratorVector *getNodeVectorForNetID(NetworkIdRef netID, bool createIfNonExistent);
	
	void addNode(TransportAddress& node, NetworkIdRef netID);
	
	void removeNode(TransportAddress& node, NetworkIdRef netID);
};
}//namespace
#endif