#ifndef __SYNAPSE_H_
#define __SYNAPSE_H_

#include <oversim_mapset.h>
#include <omnetpp.h>
#include <TransportAddress.h>
#include <OverlayKey.h>


// OverlayParametersList indexes
#define OVERLAY_TYPE		0
#define OVERLAY_NAME		1  
#define OVERLAY_NETID		2
#define OVERLAY_SYN_MODULE	3

namespace oversim {
	// The identifier for the networkID // TODO: decide if we use simply string or something else (i.e. directly the OverlayKey *)
	typedef std::string NetworkIdRef;
	// A text field representing one of the configurationparameters for an overlay
	typedef std::string OverlayParameter;
	// A list of configuration parameters for a given overlay
	typedef std::vector< OverlayParameter *> OverlayParametersList;
	// Map (and map entry) containing parameter lists for each available overlay in the system (identified by the netID)
	typedef std::pair< OverlayParameter, OverlayParametersList *> OverlayParameterMapEntry;
	typedef UNORDERED_MAP< OverlayParameter , OverlayParametersList *> OverlaysParametersMap;
	// Map (and map entry) of overlay modules to which each node is connected to
	typedef std::pair< NetworkIdRef, cModule * > ConnectedOverlaysMapEntry;
	typedef UNORDERED_MAP< NetworkIdRef, cModule * > ConnectedOverlaysMap;
	//Used by the ReverseNetworkTable
	typedef std::set<TransportAddress> SynapseNodeSet;
	typedef std::map<NetworkIdRef, SynapseNodeSet *> SynapseNodeMap;
	typedef std::vector<SynapseNodeSet::iterator> NodeIteratorVector;
	typedef std::map<NetworkIdRef, NodeIteratorVector *> NodeIteratorMap;
	//Used by the  SynapseRoutingTable
	// Counts how many times the request has been seen in a given overlay
	typedef struct {
		OverlayKey netID;
		int numTimes;
	} RoutingHistoryEntry;
	// Routing strategy type
	typedef enum {
		N_FLOOD,
		N_RANDOM_WALK,
		DIRECT
	} RoutingStrategyType;
	// Routing Strategy information, to be stored and exchanged in synapse messages
	typedef struct {
		RoutingStrategyType routingStrategy;
		int nReplicas;
		float opportunisticProbability; //probability of sending a SYN_REQ when node offers (0=no, 1=always)
		float directProbability; //probability of sending a SYN_REQ upon new routing
		std::vector<NetworkIdRef> targetOverlays;
		int ttl;
		std::map<OverlayKey, int> traversedOverlays;
	} RoutingStrategyInformation;
	typedef struct {
		std::set<TransportAddress> incomingNodes; //might have to add an RPC nonce
		std::set<TransportAddress> outgoingNodes; //might have to add an RPC nonce
	} ConnectionTable;
	// Request record: contains the current TTL for a given request, the nodes to forward it back,
	// the request history (useful to decide the routing strategy and the synapse nodes to which
	// the request has been forwarded to (entry can be cancelled when first good message arrives or
	// after a consensus...
	typedef struct {
		OverlayKey requestID;
		RoutingStrategyInformation routingState;
		ConnectionTable connections;
	} SynapseTableRequestRecord;
	// Maps a set of different request for the same key to the key
	// This way upon the first response, we can serve all
	typedef std::map<OverlayKey, std::set<SynapseTableRequestRecord> > SynapseTableRequestMap;
}

#endif